;Copyright 2000 - Mark McDonald All rights reserved
; DECLARE: SUB INT86(BYVAL integer, InRegs AS RegType, OutRegs AS RegType)
; DESC:    Clone of QuickBASIC's INTX procedure.
; EXAMP:   INT86 &H21, InRegs, OutRegs
; INT86(INERRUPT,INREG,OUTREG)
;     Allows you to execute a INTERRUPT.  Parms are passed in INREG and
;     results returned in OUTREG.  This function does NOT work within
;     units (PBU) but may be placed inside function/subs of the main
;     program.
;     WARNING:  Do not use this function if you are not totally
;               conversant with C or machine language programming.
;               Improper use may cause data loss and computer lock-up!
;
;     EXAMPLE:  INREG.H.AH = &H33
;               INREG.H.AL = &H01
;               INREG.H.DL = &H01
;               INT86 &H21,INREG,OUTREG
;     EXAMPLE:  INREG.H.AH = &H33
;               INREG.H.AL = &H00
;               INT86 &H21,INREG,OUTREG
;               T = OUTREG.H.DL     '1'
;
;     EXAMPLE:  FUNCTION SETBK(BYVAL F AS INTEGER) AS INTEGER
;                 DIM INREG AS REGS
;                 DIM OUTREG AS REGS
;                 INREG.H.AH = &H33
;                 INREG.H.AL = &H01
;                 INREG.H.DL = F
;                 INT86 &H21,INREG,OUTREG
;                 INREG.H.AH = &H33
;                 INREG.H.AL = &H00
;                 INT86 &H21,INREG,OUTREG
;                 FUNCTION = OUTREG.H.DL
;               END FUNCTION

Regs    STRUC
        RegAX   DW ?
        RegBX   DW ?
        RegCX   DW ?
        RegDX   DW ?
        RegBP   DW ?
        RegSI   DW ?
        RegDI   DW ?
        RegFL   DW ?
        RegDS   DW ?
        RegES   DW ?
Regs    EndS

MCODE   Segment Byte
        Assume  CS: MCODE

BPSave  DW      ?

        Public  INT86

INT86 Proc Far

ARG     OutRegs: DWORD, InRegs: DWORD, IntNum: WORD = Retbytes

        push    BP                      ;
        mov     BP, SP                  ;
        push    DS                      ;

        mov     CS: BPSave, BP          ; save BP

        mov     AX, IntNum              ; get interrupt number
        mov     CS: IntX, AL            ; put it in place
        jmp     Short $+2               ; flush the pre-fetch queue

        lds     SI, InRegs              ; point DS:SI to InRegs

        mov     AX, DS: [SI].RegFL      ; get flags value
        push    AX                      ;
        mov     AX, DS: [SI].RegES      ; get ES value
        push    AX                      ;
        mov     AX, DS: [SI].RegDS      ; get DS value
        push    AX                      ;
        mov     AX, DS: [SI].RegDI      ; get DI value
        push    AX                      ;
        mov     AX, DS: [SI].RegSI      ; get SI value
        push    AX                      ;
        mov     AX, DS: [SI].RegBP      ; get BP value
        push    AX                      ;
        mov     DX, DS: [SI].RegDX      ; get DX value
        mov     CX, DS: [SI].RegCX      ; get CX value
        mov     BX, DS: [SI].RegBX      ; get BX value
        mov     AX, DS: [SI].RegAX      ; get AX value
        pop     BP                      ; set BP
        pop     SI                      ; set SI
        pop     DI                      ; set DI
        pop     DS                      ; set DS
        pop     ES                      ; set ES
        popf                            ; set flags

        DB      0CDh                    ; call
IntX    DB      0                       ;  interrupt

        cmp     CS: IntX, 25h           ; Interrupts 25h and 26h
        je      SkipFlags               ;  leave the flags on the
        cmp     CS: IntX, 26h           ;  stack, so don't save them
        je      SkipFlags               ;  if we're calling them.
        pushf                           ; save flags
SkipFlags:
        push    ES                      ; save ES
        push    DS                      ; save DS
        push    SI                      ; save SI
        push    DI                      ; save DI
        push    BP                      ; save BP

        mov     BP, CS: BPSave          ; restore BP
        lds     SI, OutRegs             ; point DS:SI to OutRegs
        mov     DS: [SI].RegAX, AX      ; store AX
        mov     DS: [SI].RegBX, BX      ; store AX
        mov     DS: [SI].RegCX, CX      ; store AX
        mov     DS: [SI].RegDX, DX      ; store AX
        pop     AX                      ;
        mov     DS: [SI].RegBP, AX      ; store BP
        pop     AX                      ;
        mov     DS: [SI].RegDI, AX      ; store DI
        pop     AX                      ;
        mov     DS: [SI].RegSI, AX      ; store SI
        pop     AX                      ;
        mov     DS: [SI].RegDS, AX      ; store DS
        pop     AX                      ;
        mov     DS: [SI].RegES, AX      ; store ES
        pop     AX                      ;
        mov     DS: [SI].RegFL, AX      ; store Flags

        pop     DS                      ;
        pop     BP                      ;
        retf    Retbytes                ;
INT86 EndP
MCODE   EndS
        End
