; DOSBOX.ASM   confines standard output to a specified rectangular
;               area of the screen, allowing QuickBASIC to run SHELL
;               commands in a window.
;
;   Author:     Christy Gemmell
;   For:        Assembly-Language Toolbox for QuickBASIC
;   Version:    5.57
;   Date:       20/4/1994
;
;   Compatible with Microsoft QuickBASIC and VisualBASIC for DOS
;   Assembled using Microsoft Macro Assembler, MASM version 6.10a
;
;   This routine takes over INT 29h, the DOS console device output or
;   FAST PUTCHAR routine which is called internally by MS-DOS if output
;   is going to a device rather than a file.
;
                .model  large
;               public  DOSBox
;               extrn   VideoType:proc
;               .code
MCODE   Segment Byte
        Assume  CS: MCODE

        Public  DOSBOX

Vector          label   dword                   ; Saved INT 29h vector
Ofset           dw      0                       ; Offset
Segmt           dw      0                       ; Segment
TopLeft         label   word                    ; Upper left co-ordinates
Left            db      1                       ; Leftmost column
Top             db      10                      ; Topmost row
BotRight        label   word                    ; Lower right co-ordinates
Right           db      79                      ; Rightmost column
Bottom          db      24                      ; Bottommost row
Attrib          db      31                      ; Display attribute
Cols            db      79                      ; Screen width in columns
Rows            db      24                      ; Screen height in rows
Pge             db      0                       ; Active display page
Posn            dw      ?                       ; Current cursor position

;   By default text which would appear outside the DOSBOX window is
;   automatically wrapped round to the next line. If this flag is set
;   to zero, however, out-of-bounds text is truncated instead.
;
Wrap            db      ?                       ; Line wrapping flag

;   Replacement INT 29h interrupt handler.
;
Handler         proc far
                sti                             ; Enable interrupts
                pushf                           ; Preserve flags
                push    ax                      ; Preserve
                push    bx                      ;  these
                push    cx                      ;   registers
                push    dx                      ;
                push    ds                      ;
                push    ax                      ; Preserve output character
                push    cs                      ; Align Code and
                pop     ds                      ;  Data segments
                mov     bh,Pge                  ; Current display page
                mov     ah,03h                  ; BIOS video service 3
                int     10h                     ;  - get cursor position
                mov     Posn,dx                 ; We might need it later
                xor     cx,cx                   ; Clear relocate flag
                cmp     dh,Top                  ; Compare with top margin
                jae     Hand_01                 ; Branch if within bounds
                mov     dx,TopLeft              ; Go to top-left corner
                mov     ah,02h                  ; BIOS video service 2
                int     10h                     ;  - set cursor position
                jmp     Hand_09                 ; Display the character
Hand_01:
                cmp     dl,Left                 ; Compare with left margin
                jae     Hand_02                 ; Branch if not below
                mov     cx,-1                   ; Setup for cursor locate
                mov     dl,Left                 ; Move to left margin
Hand_02:
                cmp     dl,Right                ; Compare with right margin
                jbe     Hand_03                 ; Branch if not above
                mov     cx,-1                   ; Set up for cursor locate
                mov     dl,Right                ; Move to beginning
                inc     dh                      ;  of next row
Hand_03:
                cmp     cx,-1                   ; Out of bounds?
                jne     Hand_07                 ; No, proceed
                cmp     byte ptr Wrap,1         ; Are we wrapping lines?
                jne     Hand_04                 ; No we're truncating
                mov     ah,02h                  ; BIOS video service 2
                int     10h                     ;  - set cursor position
                xor     cx,cx                   ; Clear bounds flag
                jmp     short Hand_07           ; Check for bottom
Hand_04:
                mov     dx,Posn                 ; Retrieve cursor position
                pop     ax                      ; Recover character
                cmp     al,7                    ; Console bell?
                je      Hand_10                 ; Let it ring!
                cmp     al,10                   ; Line feed?
                jne     Hand_05                 ; No, try again
                inc     dh                      ; Jump to
                jmp     short Hand_06           ;  next row
Hand_05:
                cmp     al,13                   ; Carriage return?
                jne     Hand_11                 ; Ignore everything else
                mov     dl,Left                 ; Return to left margin
Hand_06:
                mov     ah,02h                  ; BIOS video service 2
                int     10h                     ;  - set cursor position
Hand_07:
                mov     al,Bottom               ; Compare with
                cmp     dh,al                   ;  bottom margin
                jbe     Hand_08                 ; Branch if not below
                push    cx                      ; Save print flag
                mov     cx,TopLeft              ; Define window
                mov     dx,BotRight             ;  margins and
                mov     bh,Attrib               ;   colour
                mov     ax,0601h                ; BIOS video service 6
                int     10h                     ;  - scroll window up
                mov     bh,Pge                  ; Current display page
                mov     dl,Left                 ; Move to beginning
                mov     dh,Bottom               ;  of bottom row
                mov     ah,02h                  ; BIOS video service 2
                int     10h                     ;  - set cursor position
                pop     cx                      ; Retrieve print flag
Hand_08:
                jcxz    Hand_09
                jmp     short Hand_11
Hand_09:
                pop     ax                      ; Retrieve output character
Hand_10:
                mov     bh,Pge                  ; Current display page
                xor     bl,bl                   ; Foreground n/a in text
                mov     ah,0Eh                  ; BIOS Video service 14
                int     10h                     ;  - write character TTY
Hand_11:
                pop     ds                      ; Clean up the stack
                pop     dx
                pop     cx
                pop     bx
                pop     ax
                popf                            ; Recover flags
                iret                            ; Return to caller
Handler         endp

;  Enable or disable DOS display window.
;
DOSBox          proc    far
                push    bp                      ; Save Base pointer
                mov     bp,sp                   ; Establish stack frame
                push    ds                      ; Save
                push    es                      ;  working
                push    di                      ;   registers
                push    cs                      ; Align Code and
                pop     ds                      ;  Data segments
                mov     al,[bp+16]              ; Open or close window?
                cmp     al,0                    ; If zero branch
                je      Box_02                  ;  to close window
                mov     byte ptr Wrap,1         ; Default to line wrapping
                cmp     al,2                    ; Unless customer
                jne     Box_01                  ;  wants 'em
                mov     byte ptr Wrap,0         ;   chopped off

;   Define DOSBox window limits and colour, install interrupt handler,
;   clear window and locate cursor in top left-hand corner.
;
Box_01:
                cmp     Segmt,0                 ; Already open?
                jnz     Box_03                  ; If so, abort
          ;     call    VideoType               ; Collect display information
          ;     mov     Cols,ah                 ; Store screen width
                mov     Cols,79                 ; Store screen width
          ;     mov     word ptr Rows,bx        ; Store height and page
                mov     word ptr Rows,24        ; Store height and page
                mov     ax,[bp+14]              ; Get top row
                mov     bx,[bp+12]              ; Get left column
                call    Adjust                  ; Adjust for screen offsets
                mov     TopLeft,bx              ; Save top-left corner
                mov     ax,[bp+10]              ; Get bottom row
                mov     bx,[bp+8]               ; Get right column
                call    Adjust                  ; Adjust for screen offsets
                mov     ax,TopLeft              ; Retrieve top-left coordinates
                cmp     ah,bh                   ; Make sure that
                jge     Box_03                  ;  top-left is above
                cmp     al,bl                   ;   and to the left of
                jge     Box_03                  ;    Bottom right
                mov     BotRight,bx             ; Save bottom-right corner
                mov     ax,[bp+6]               ; Get attribute
                mov     Attrib,al               ;  and save it
                mov     ax,3529h                ; Get and
                int     21h                     ;  save the
                mov     Ofset,bx                ;   old INT 29h
                mov     Segmt,es                ;    vector
                mov     dx,offset Handler       ; Reset INT 29h to
                mov     ax,2529h                ;  point to our own
                int     21h                     ;   interrupt handler
                mov     cx,TopLeft              ; Scroll
                mov     dx,BotRight             ;  DOS Box
                mov     bh,Attrib               ;   screen
                mov     ax,0600h                ;    window
                int     10h                     ;     clear
                mov     bh,Pge                  ; Move the
                mov     dx,TopLeft              ;  cursor to
                mov     ah,02h                  ;   top-left
                int     10h                     ;    corner
                jmp     short Box_03            ; Branch to exit

;   Restore old INT 29h vector and disable DOSBox processing.
;
Box_02:
                cmp     Segmt,0                 ; Is handler installed?
                je      Box_03                  ; If not ignore call
                lds     dx,Vector               ; Restore
                mov     ax,2529h                ;  original
                int     21h                     ;   vector
                xor     ax,ax                   ; Clear stored
                mov     Ofset,ax                ;  vector
                mov     Segmt,ax                ;   flags
Box_03:
                pop     di                      ; Clean
                pop     es                      ;  up
                pop     ds                      ;   the
                pop     bp                      ;    stack
                ret     12                      ; Return to QuickBASIC
DOSBox          endp

;   Convert row-column co-ordinates.
;
;   Combine row and column parameters into single byte values and adjust
;   values to ROM-BIOS numbering system and actual screen dimensions.
;
Adjust          proc    near
                mov     bh,al                   ; Row to BH
                cmp     bh,0                    ; Don't go
                jg      Adj_01                  ;  over the
                mov     bh,1                    ;   top!
Adj_01:
                cmp     bl,0                    ; Don't fall off
                jg      Adj_02                  ;  the left-hand
                mov     bl,1                    ;   side
Adj_02:
                cmp     bh,Rows                 ; Don't drop
                jb      Adj_03                  ;  off the
                mov     bh,Rows                 ;   bottom
Adj_03:
                cmp     bl,Cols                 ; Don't fall off
                jb      Adj_04                  ;  the right-hand
                mov     bl,Cols                 ;   side
Adj_04:
                sub     bx,0101h                ; Use BIOS numbering
                ret
Adjust          endp

MCODE   EndS
        End
                end
